@@ -25,20 +25,28 @@ class WebSocketClientProtocol(WebSocketCommonProtocol):
     state = 'CONNECTING'
 
     @asyncio.coroutine
-    def handshake(self, uri):
+    def handshake(self, wsuri, origin=None):
         """
         Perform the client side of the opening handshake.
+
+        If provided, ``origin`` sets the HTTP Origin header.
         """
-        # Send handshake request. Since the uri and the headers only contain
-        # ASCII characters, we can keep this simple.
-        request = ['GET %s HTTP/1.1' % uri.resource_name]
-        set_header = lambda k, v: request.append('{}: {}'.format(k, v))
-        if uri.port == (443 if uri.secure else 80):         # pragma: no cover
-            set_header('Host', uri.host)
+        headers = []
+        set_header = lambda k, v: headers.append((k, v))
+        if wsuri.port == (443 if wsuri.secure else 80):         # pragma: no cover
+            set_header('Host', wsuri.host)
         else:
-            set_header('Host', '{}:{}'.format(uri.host, uri.port))
+            set_header('Host', '{}:{}'.format(wsuri.host, wsuri.port))
+        if origin is not None:
+            set_header('Origin', origin)
         set_header('User-Agent', USER_AGENT)
         key = build_request(set_header)
+        self.raw_request_headers = headers
+
+        # Send handshake request. Since the URI and the headers only contain
+        # ASCII characters, we can keep this simple.
+        request = ['GET %s HTTP/1.1' % wsuri.resource_name]
+        request.extend('{}: {}'.format(k, v) for k, v in headers)
         request.append('\r\n')
         request = '\r\n'.join(request).encode()
         self.writer.write(request)
@@ -50,6 +58,7 @@ class WebSocketClientProtocol(WebSocketCommonProtocol):
             raise InvalidHandshake("Malformed HTTP message") from exc
         if status_code != 101:
             raise InvalidHandshake("Bad status code: {}".format(status_code))
+        self.raw_response_headers = list(headers.raw_items())
         get_header = lambda k: headers.get(k, '')
         check_response(get_header, key)
 
@@ -59,12 +68,14 @@ class WebSocketClientProtocol(WebSocketCommonProtocol):
 
 @asyncio.coroutine
 def connect(uri, *,
-            klass=WebSocketClientProtocol, **kwds):
+            klass=WebSocketClientProtocol, origin=None, **kwds):
     """
     This coroutine connects to a WebSocket server.
 
-    It's a thin wrapper around the event loop's ``create_connection`` method.
-    Extra keyword arguments are passed to ``create_server``.
+    It accepts an ``origin`` keyword argument to set the Origin HTTP header.
+
+    It's a thin wrapper around the event loop's `create_connection` method.
+    Extra keyword arguments are passed to `create_server`.
 
     It returns a :class:`~websockets.client.WebSocketClientProtocol` which can
     then be used to send and receive messages.
@@ -80,15 +91,19 @@ def connect(uri, *,
     Connection" in RFC 6455, except for the requirement that "there MUST be no
     more than one connection in a CONNECTING state."
     """
-    uri = parse_uri(uri)
-    kwds.setdefault('ssl', uri.secure)
+    wsuri = parse_uri(uri)
+    if wsuri.secure:
+        kwds.setdefault('ssl', True)
+    elif 'ssl' in kwds:
+        raise ValueError("connect() received a SSL context for a ws:// URI. "
+                         "Use a wss:// URI to enable TLS.")
+    factory = lambda: klass(host=wsuri.host, port=wsuri.port, secure=wsuri.secure)
     transport, protocol = yield from asyncio.get_event_loop().create_connection(
-            klass, uri.host, uri.port, **kwds)
+            factory, wsuri.host, wsuri.port, **kwds)
 
     try:
-        yield from protocol.handshake(uri)
+        yield from protocol.handshake(wsuri, origin=origin)
     except Exception:
-        protocol.writer.write_eof()
         protocol.writer.close()
         raise
 
