@@ -79,6 +79,7 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
         # Futures tracking steps in the connection's lifecycle.
         self.opening_handshake = asyncio.Future()
         self.closing_handshake = asyncio.Future()
+        self.connection_failed = asyncio.Future()
         self.connection_closed = asyncio.Future()
 
         # Queue of received messages.
@@ -157,12 +158,14 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
             pass
 
         # Wait for a message until the connection is closed
-        next_message = asyncio.Task(self.messages.get())
+        next_message = asyncio.async(self.messages.get())
         done, pending = yield from asyncio.wait(
                 [next_message, self.worker],
                 return_when=asyncio.FIRST_COMPLETED)
         if next_message in done:
             return next_message.result()
+        else:
+            next_message.cancel()
 
     @asyncio.coroutine
     def send(self, data):
@@ -230,6 +233,8 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
                 break
             except WebSocketProtocolError:
                 yield from self.fail_connection(1002)
+            except asyncio.IncompleteReadError:
+                yield from self.fail_connection(1006)
             except UnicodeDecodeError:
                 yield from self.fail_connection(1007)
             except PayloadTooBig:
@@ -305,7 +310,8 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
                     # 7.1.3. The WebSocket Closing Handshake is Started
                     self.state = 'CLOSING'
                     yield from self.write_frame(OP_CLOSE, frame.data, 'CLOSING')
-                self.closing_handshake.set_result(True)
+                if not self.closing_handshake.done():
+                    self.closing_handshake.set_result(True)
                 return
             elif frame.opcode == OP_PING:
                 # Answer pings.
@@ -346,9 +352,11 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
             # Handle flow control automatically.
             yield from self.writer.drain()
         except ConnectionResetError:
-            # Terminate the connection if the socket died.
-            self.state = 'CLOSING'
-            yield from self.fail_connection(1006)
+            # Terminate the connection if the socket died,
+            # unless it's already being closed.
+            if expected_state != 'CLOSING':
+                self.state = 'CLOSING'
+                yield from self.fail_connection(1006)
 
     @asyncio.coroutine
     def close_connection(self):
@@ -389,6 +397,15 @@ class WebSocketCommonProtocol(asyncio.StreamReaderProtocol):
 
     @asyncio.coroutine
     def fail_connection(self, code=1011, reason=''):
+        # Avoid calling fail_connection more than once to minimize
+        # the consequences of race conditions between the two sides.
+        if self.connection_failed.done():
+            # Wait until the other coroutine calls connection_lost.
+            yield from self.connection_closed
+            return
+        else:
+            self.connection_failed.set_result(None)
+
         # Losing the connection usually results in a protocol error.
         # Preserve the original error code in this case.
         if self.close_code != 1006:
